@@ -1,11 +1,13 @@
 from __future__ import annotations
 
 import base64
+import bz2
 import contextlib
 import functools
 import glob
 import gzip
 import inspect
+import lzma
 import os
 import pathlib
 import pickle
@@ -21,7 +23,7 @@ from itertools import chain
 from os.path import basename, dirname, expanduser, isfile
 from os.path import join as jj
 from os.path import normpath, splitext
-from typing import Any, Callable, List, Sequence, Set, TypeVar
+from typing import Any, Callable, List, Literal, Sequence, Set, TypeVar
 from urllib.error import URLError
 from urllib.request import urlretrieve
 
@@ -102,7 +104,7 @@ def ts_data_path(directory: str, filename: str):
 
 def data_path_ts(directory: str, path: str):
     name, extension = splitext(path)
-    return jj(directory, '{}_{}{}'.format(name, time.strftime("%Y%m%d%H%M"), extension))
+    return jj(directory, f'{name}_{time.strftime("%Y%m%d%H%M")}{extension}')
 
 
 def path_add_sequence(path: str, i: int, j: int = 0) -> str:
@@ -286,7 +288,7 @@ def parse_range_list(rl):
         if len(parts) == 0:
             return range(0, 0)
         if len(parts) > 2:
-            raise ValueError("Invalid range: {}".format(r))
+            raise ValueError(f"Invalid range: {r}")
         return range(parts[0], parts[-1] + 1)
 
     ranges = sorted(set(map(parse_range, rl.split(","))), key=lambda x: (x.start, x.stop))
@@ -349,3 +351,41 @@ def dedent(text: str) -> str:
 def is_empty(filename: str) -> bool:
     """Check if file is empty."""
     return os.path.exists(filename) and os.stat(filename).st_size == 0
+
+
+def strip_csv_header(csv_str: str, sep: str = '\n') -> str:
+    """Remove header line from `csv_str`"""
+    if not csv_str:
+        return ''
+    idx: int = csv_str.find(sep)
+    if idx <= 0:
+        return ''
+    return csv_str[idx + 1 :]
+
+
+def merge_tagged_csv(csv_strings: List[str], sep: str = '\n') -> str:
+    """Merge tagged CSV strings into a single tagged CSV string"""
+    if len(csv_strings or []) == 0:
+        return ''
+    texts: List[str] = [csv_strings[0]]
+    for csv_string in csv_strings[1:]:
+        text = strip_csv_header(csv_string, sep=sep)
+        if text != '':
+            texts.append(text)
+    return sep.join(texts)
+
+
+def store_to_compressed_file(filename: str, text: str, target_type: Literal['plain', 'gzip', 'bz2', 'lzma']) -> None:
+    """Stores a textfile on disk - optionally compressed"""
+    modules = {'gzip': (gzip, 'gz'), 'bz2': (bz2, 'bz2'), 'lzma': (lzma, 'xz')}
+
+    if target_type in modules:
+        module, extension = modules[str(target_type)]
+        with module.open(f"{filename}.{extension}", 'wb') as fp:
+            fp.write(text.encode('utf-8'))
+
+    elif target_type == 'plain':
+        with open(filename, 'w', encoding='utf-8') as fp:
+            fp.write(text)
+    else:
+        raise ValueError(f"unknown mode {target_type}")
